'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.rulesCreator = rulesCreator;
exports.factoryValidationObj = factoryValidationObj;
exports.findIndexForActiveError = findIndexForActiveError;
exports.isErrorActive = isErrorActive;
exports.appRulesValidation = appRulesValidation;

var _lodash = require('lodash.get');

var _lodash2 = _interopRequireDefault(_lodash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * @typedef {{valid: Boolean, errors: [String] }} validationsObj
 */

/**
 * @typedef {{valid: Boolean, error: String }} validationObj
 */

/**
 * @typedef {{ type: String, name: String, stateMap: String, compareWith: any, defaultValue: any }} IDescription
 */

/**
 * @typedef {{ errorType: String, validate: Function }} IRulesDescriptor
*/

var matchKey = function matchKey(key) {
  return function (t) {
    return t.indexOf(key) === 0;
  };
};

var isErrorActiveMaching = function isErrorActiveMaching(rulesErrors, type) {
  return typeof type === 'string' ? rulesErrors.findIndex(matchKey(type)) : type.findIndex(function (t) {
    return rulesErrors.findIndex(matchKey(t)) >= 0;
  });
};

var isErrorAbsolute = function isErrorAbsolute(rulesErrors, type) {
  return typeof type === 'string' ? rulesErrors.indexOf(type) : type.findIndex(function (t) {
    return rulesErrors.indexOf(t) >= 0;
  });
};

var capitalizeFirstLetter = function capitalizeFirstLetter(string) {
  return string.charAt(0).toUpperCase() + string.slice(1);
};

var capitalize = function capitalize(string) {
  return string.trim().replace(/-/, ' ').split(' ').map(capitalizeFirstLetter).reduce(function (reduced, value) {
    return '' + reduced + value;
  }, '');
};

/**
 * Pass array of rules
 * @param {Array}
 * @param {Object}
 * @param {Function}
 * @return {Object} { valid: Boolean, errors: [String] }
 */
function validate() {
  var rules = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
  var state = arguments[1];
  var previus = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : { valid: true, errors: [] };

  return Object.keys(rules).reduce(function (results, key) {
    var rule = rules[key];
    var appliedRule = rule(state);
    results.valid = results.valid && appliedRule.valid;
    if (appliedRule.error) {
      results.errors.push(appliedRule.error);
    }
    return results;
  }, previus);
}

/**
 * Validate if patient info is complete
 * @param {IDescription} description
 * @returns {IRulesDescriptor}
*/
function ruleCreator(validators, description) {
  var errorType = description.name + '-' + description.type;
  var validate = validators[description.type] ? validators[description.type](description.stateMap, errorType, description.compareWith, description.defaultValue) : validators.required(description.stateMap, errorType, description.compareWith, description.defaultValue);
  return {
    validate: validate,
    errorType: errorType
  };
}

/**
 * Validate if patient info is complete
 * @param {Object} types
 * @param {Array<IDescription>} descriptions
 * @returns {Object}
*/
function rulesCreator(validators) {
  var types = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var descriptions = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [];

  return descriptions.reduce(function (rules, description) {
    if (!description.name || !description.type || !description.stateMap) {
      throw new Error('Every descriptor needs at least name, type and stateMap');
    }

    var _ruleCreator = ruleCreator(validators, description),
        validate = _ruleCreator.validate,
        errorType = _ruleCreator.errorType;

    var typeConst = errorType.toUpperCase().replace(/-/g, '_');
    var functionName = capitalize(errorType);
    rules[functionName] = validate;
    types[typeConst] = errorType;
    return rules;
  }, {});
}

/**
 * Create a validation object
 * @param {Boolean} valid
 * @param {String} errorType
 * @returns {validationObj}
 */
function factoryValidationObj() {
  var valid = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;
  var errorType = arguments[1];

  return {
    valid: Boolean(valid),
    error: valid ? undefined : errorType
  };
}

function findIndexForActiveError() {
  var rulesErrors = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
  var types = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';
  var match = arguments[2];

  return match ? isErrorActiveMaching(rulesErrors, types) : isErrorAbsolute(rulesErrors, types);
}

function isErrorActive() {
  var rulesErrors = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
  var types = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';
  var match = arguments[2];

  return findIndexForActiveError(rulesErrors, types, match) >= 0;
}

/**
 * Pass validation rules per application
 * @param {Object} rules
 * @param {Object} store application store
 * @param {String} appName smartgut|smartjane|smartflu
 * @param {String} component validation component name
 * @return {validationsObj}
 */
function appRulesValidation(rules, state) {
  var appName = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : '';
  var component = arguments[3];

  var generalRules = (0, _lodash2.default)(rules, 'default.' + component);
  var appRules = (0, _lodash2.default)(rules, appName.toLowerCase() + '.' + component);
  var generals = generalRules ? validate(generalRules, state) : undefined;
  return !appRules ? generals : validate(appRules, state, generals);
}